from rocolib.api.components.Component import Component

c = Component()

#Add brain and wheels parameter (Default: Adafruit ESP32 2 Stacks)
c.addParameter("brains", "esp32stack", paramType="dimension")
c.addParameter("driveservo", "fs90r", paramType="dimension")

#The height will always match the height of the brains
c.addParameter("length", 80, paramType="length", minValue=80) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=60) #tested minValue
c.addParameter("steerangle", 3.14, paramType="length") #Change this to change direction of car

for i in range(2):
    c.addSubcomponent("car%d" % i, "HalfCar")
    c.addConstraint(("car%d" % i, "length"), ("length", "driveservo", "brains"), "(x[0] - (x[0]/2. - 2 * getDim(x[1], 'hornoffset')"
                                                                                 "- getDim(x[2], 'width')))")
    c.addConstraint(("car%d" % i, "height"), "brains", '(getDim(x, "height"))')
    c.addConstraint(("car%d" % i, "width"), "width")

#Main component of steering wheel car which makes them moveable: trapezoid that connects the two half of the car
for i in range(4):
    c.addSubcomponent("midsupport%d" %i, "Trapezoid")
    c.addConstraint(("midsupport%d" %i, "width"), "width", "x-(2*x*.15)")
    c.addConstraint(("midsupport%d" %i, "depth"), "driveservo", "getDim(x, 'motorlength')")
    c.addConstraint(("midsupport%d" %i, "bangle"), ("width", "driveservo"),
                    "float(np.atan(getDim(x[1], 'motorlength')/(x[0]*.15))* 180 * 0.318309)")

c.addConnection(("car1", "rect4.r"), ("midsupport0", "botedge"))
c.addConnection(("car0", "rect4.r"), ("midsupport1", "botedge"))

c.addConnection(("car1", "rect3.r"), ("midsupport2", "botedge"))
c.addConnection(("car0", "rect3.r"), ("midsupport3", "botedge"))

#Putting Brains
c.addSubcomponent("holder", "ESP32Stack")
c.addConstraint(("holder", "length"), "width")

c.addSubcomponent("holdersplit", "SplitEdge")
c.addConstraint(("holdersplit", "toplength"), "brains", '(getDim(x, "height"),)')
c.addConstraint(("holdersplit", "botlength"),
                ("driveservo", "brains"),
                "(getDim(x[0], 'motorwidth'), getDim(x[1], 'height') - getDim(x[0], 'motorwidth'))")

c.addConnection(("holdersplit", "topedge0"), ("holder", "cover.topedge1"))
c.addConnection(("car0", "bright.topedge0"), ("holdersplit", "botedge0"), angle=-90)

#Putting servo hinge between cars
c.addSubcomponent("hinge0", "Pivot", inherit="servo.angle")
c.addConstraint(("hinge0", "length"), "length")
c.addConstraint(("hinge0", "width"), "width")
c.addSubcomponent("hingeSplit0", "SplitEdge")
c.addConstraint(("hingeSplit0", "toplength"), "brains", '(getDim(x, "height"), )')
c.addConstraint(("hingeSplit0", "botlength"), ("brains", "driveservo"),
                    '((getDim(x[0], "height")-getDim(x[1], "motorheight"))/2, getDim(x[1], "motorheight"), '
                    '(getDim(x[0], "height")-getDim(x[1], "motorheight"))/2)')

c.addConnection(("car0", "rect2.l"), ("hingeSplit0", "topedge0"))
c.addConnection(("hingeSplit0", "botedge1"), ("hinge0", "belt2.r"), angle=-90)

#Attach hinge on face of Trapezoid to move in Webots
c.addConnection(("midsupport2", "topedge"), ("hinge0", "pivotface.topedge"), angle=180)
c.addConstraint(("hinge0", "servo.angle"), "steerangle")

c.toLibrary("PivotCar")
